'''
Created on Jun 24, 2010

@author: thebwt
'''
import pyglet

from twisted.internet import reactor
from twisted.spread import pb


from wengine.modes import BaseMode
from base.models.widgets import WindowWidget, ButtonWidget, FieldWidget, FormWidget, InputWidget


class ChatMode(BaseMode):

	def __init__(self, host='localhost', port = 8800, nick='default', **kwargs):
		super(ChatMode, self).__init__()
		self.nick = nick
		self.buf = pyglet.text.document.UnformattedDocument()
		self.log = pyglet.text.document.UnformattedDocument()
		self.bufLayout = pyglet.text.layout.IncrementalTextLayout(self.buf, 400, 20)
		self.logLayout = pyglet.text.layout.IncrementalTextLayout(self.log, 400, 580, multiline=True)
		self.logLayout.x, self.logLayout.y =0,20
		self.caret = pyglet.text.caret.Caret(self.bufLayout)
		self.caret2 = pyglet.text.caret.Caret(self.logLayout)
		self.caret.filtered = self.caret.on_text
		self.caret.on_text = self.return_filter
		factory = pb.PBClientFactory()
		reactor.connectTCP(host, port, factory)
		factory.getRootObject().addCallback(self.setServer)

	def setServer(self, obj):
		server = obj
		rel = pb.Referenceable()
		rel.remote_update = self.write_to_log
		server.callRemote("ident", rel, self.nick).addCallback(self.setAvatar)

	def setAvatar(self, obj):
		self.avatar = obj

	def on_draw(self):
		super(ChatMode, self).on_draw()
		self.bufLayout.draw()
		self.logLayout.draw()

	def start(self):
		super(ChatMode, self).start()
		for win in pyglet.app.windows:
			win.push_handlers(self.caret)

	def on_key_press(self, symbol, modifiers):
		if symbol == pyglet.window.key.ENTER:
			self.avatar.callRemote("say", self.buf.text)
			#self.write_to_log(self.buf.text)
			self.bufLayout.begin_update()
			self.buf.text = ''
			self.bufLayout.end_update()

	def return_filter(self, text):
		if ord(text) == 13:
			return False
		else:
			self.caret.filtered(text)

	def write_to_log(self, nick, msg):
		self.logLayout.begin_update()
		self.log.text += nick +" : " +msg + '\n'
		self.logLayout.end_update()

class Chatter(pb.Referenceable):
	"""Chat avatar"""

	def __init__(self, nick, server, buf):
		self.nick = nick
		self.server = server
		self.buf = buf

	def remote_say(self, msg):
		self.server.relay(self.nick, msg)
		return True

class ChatServer(pb.Root):
	"""Server for chatting"""

	def __init__(self):
		self.connections = []

	def remote_ident(self, buf, nick):
		self.connections.append(Chatter(nick, self, buf))
		return self.connections[-1]

	def relay(self, nick, msg):
		for connection in self.connections:
			connection.buf.callRemote("update", nick, msg)

def host_mode(Nickname='Admin'):
	reactor.listenTCP(8800, pb.PBServerFactory(ChatServer()))
	for window in pyglet.app.windows:
		window.dispatch_event('mode_change', ChatMode(nick=Nickname))

def connect_mode(Address="localhost", Nickname="Default"):
	for window in pyglet.app.windows:
		window.dispatch_event('mode_change', ChatMode(host=Address, nick=Nickname))

def host_window():
	bob = FormWidget(title = "Chat Connect", func = host_mode)
	bob.add_field(InputWidget(name="Nickname", default="Admin"))
	return bob

def connect_window():
	bob = FormWidget(title = "Chat Connect", func = connect_mode)
	bob.add_field(InputWidget(name="Address", default="localhost"))
	bob.add_field(InputWidget(name="Nickname", default="Newbie"))
	return bob
