import pyglet
from pyglet.graphics import OrderedGroup
from pyglet.text import Label

from wengine.modes import BaseMode
from base.models.widgets import WindowWidget, WindowManager, FieldWidget

class RootMode(BaseMode):
	"""
	TODO:
		Base Window, the mode's "feature".
		-RootMode(classic) does this well.
		-This mode should take an iterator of options, options being a\ 
			name/function pair.
	"""
	def __init__(self, title="Wengine Base", options=()):
		super(RootMode, self).__init__()
		self.buttons = []
		self.draw_title(title)
		self.draw_options(options)
		self.over = None
		self.wm = WindowManager((200,200))

	def draw_options(self, options):
		#get window height for starting y
		w, h = self.win.get_size()
		x = 25
		y= h/2
		for opt in options:
			self.buttons.append(Label(
				opt[0], 
				color = (0,0,0,255), 
				x = x,
				y = y,
				batch = self.feature, 
				group = self.featureGroup, 
				anchor_y = 'top', 
				bold = True
				))
			self.buttons[-1].func = opt[1]
			self.buttons[-1].defColor = (0,0,0,255)
			self.buttons[-1].ovColor = (255,0,0,255)
			y-=self.buttons[-1].content_height

	def draw_title(self, title):
		Label(
		title, 
		color = (0,0,0,255), 
		anchor_x = 'right', 
		anchor_y = 'top', 
		y =self.win.height-25, x = self.win.width-25, 
		font_size = 32, batch = self.feature, group = self.featureGroup)

	def pos_to_button(self, x, y):
		for k in self.buttons:
			if (y < k.y) and (y > (k.y - k.content_height)):
				if (x > k.x) and (x < (k.x + k.content_width)):
					return k

	def on_mouse_press(self, x, y, *args):
		if self.wm.top_window_at((x,y)): return False
		z = self.pos_to_button(x,y)
		if z and z.func: 
			if isinstance(z.func, WindowWidget):
				self.wm.add(z.func)
			else:
				return z.func()
		else: 
			return False

	def on_mouse_motion(self,x,y,dx,dy):
		if self.wm.top_window_at((x,y)):
			if self.over:
				self.over.color = self.over.defColor
				self.over = None
			return False
		p = self.pos_to_button(x,y)
		if p:
			if self.over:
				if self.over == p:
					return
				else:
					self.over.color = self.over.defColor
					self.over = p
					self.over.color = self.over.ovColor
			else:
				self.over = p
				self.over.color = self.over.ovColor
		else:
			if self.over:
				self.over.color = self.over.defColor
				self.over = None

	def start(self):
		for window in pyglet.app.windows:
			window.push_handlers(self.wm)

	def close(self):
		for window in pyglet.app.windows:
			window.pop_handlers()

