'''
Created on Apr 3, 2010

@author: thebwt
'''
from pyglet import app
from pyglet.event import EVENT_HANDLED, EVENT_UNHANDLED
from pyglet.text import Label

from twisted.internet import reactor
from twisted.spread import pb

from wengine.widgets.menus import WindowMenu, WindowForm
from wengine.widgets.buttons import ButtonList
from wengine.modes import BaseMode

from base.modes.game import GameMode, window
from base.modes.chat import ChatMode, ChatServer
from base.modes.newRoot import RootMode as RootModeTwo

class RootMode(BaseMode):
	'''
	An implementation of a 'Main Menu' custom mode.

	This mode is used to set up to access other modes and to do 
	configuration. This is primarily an example.

	This mode heavily uses buttonLists and children to manage 
	collections of buttons and menus.

	children - holds all the drawable contents of the Mode. These 
	'children' should implement a draw() function.
	
	buttonLists -  holds all the clickable contents of the mode. 
	These should implement a .func() functioni. They are generally 
	generated from wengine.models.buttons.

	functions to be attached to buttons should be created here 
	and passed to a button maker.
	'''

	def __init__(self):
		'''
		No parameters yet!
		'''
		super(RootMode, self).__init__()

		self.children.append(Label(
			"Old Wengine Base", 
			color = (0,0,0,255), 
			anchor_x = 'right', 
			anchor_y = 'top', 
			x =775, y = 575, 
			font_size = 32))
		buttons = (("New Game", self.newGame),
				("Quick Game", self.gameMode),
				("Chat Connect", self.chatConnectW),
				("Chat Host", self.chatHost),
				("Widgets Mode", self.widgetsMode),
				("Quit", self.quit),)

		blist = ButtonList((25,300),buttons)
		self.buttonLists.append(blist)
		self.children.append(blist)
		
		self.over=None
		

	def newGame(self):
		rows = (
		       (("label", ("Player Name",)),),
		       (("separator",),),
		       (("button", ("Cancel", "close-button")),("label", ("      ",)),("button", ("Start!", self.gameMode)),),
		       )
		self.addWindow(WindowMenu("New Game",rows, pos = (300,400), parent = self))

	def chatConnectW(self):
		rows = (
		       (("field", ("nick",)),),
		       (("separator",),),
		       (("button", ("Cancel", "close-button")),("label", ("      ",)),("button", ("Connect", "form-submit")),),
		       )
		self.addWindow(WindowForm("Connect to Chat",self.chatConnect, rows, pos = (300,400), parent = self))
		
	def gameMode(self):
		for window in app.windows:
			window.dispatch_event('mode_change', GameMode())

	def widgetsMode(self):
		for win in app.windows:
			win.dispatch_event('mode_change', RootModeTwo("Wengine Base", (("New Game", window()),)))

	def chatHost(self,**kwargs):
		reactor.listenTCP(8800, pb.PBServerFactory(ChatServer()))
		for window in app.windows:
			window.dispatch_event('mode_change', ChatMode(**kwargs))

	def chatConnect(self, **kwargs):
		for window in app.windows:
			window.dispatch_event('mode_change', ChatMode(**kwargs))

	def addWindow(self, child):
		self.children.append(child)
		self.buttonLists.append(child.buttons)

	def closeWindow(self, child):
		self.children.remove(child)
		self.buttonLists.remove(child.buttons)

	def on_draw(self):
		super(RootMode, self).on_draw()
		for child in self.children:
			child.draw()

	def on_mouse_press(self,x,y,buttons,mods):
		ret = EVENT_UNHANDLED
		if buttons == 1:
			for buttonList in self.buttonLists:
				button = buttonList.posToButton(x,y)
				if button: #TODO: handle window stacks ...somewhere else.
					button.func()
				ret = EVENT_HANDLED
		return ret
		
	def on_mouse_motion(self,x,y,dx,dy):
		ret = EVENT_UNHANDLED
		for buttonList in self.buttonLists:
			button = buttonList.posToButton(x,y)
			if button: #if over a button
				#print "over a button",button.text, x, y
				if (button) and (self.over == button): #if that button is the same as the old button
					return
				else: #must be over something new
					if self.over: self.over.color = self.over.defaultColor
					self.over = button
					self.over.color = self.over.overColor
					ret = EVENT_HANDLED
					break
			else:# not over a button
				#print "not over a button", x, y
				if self.over: self.over.color = self.over.defaultColor
				self.over = None
		return ret

	def quit(self):
		for window in app.windows:
			window.dispatch_event('on_close')
