'''
Created on Apr 2, 2010

@author: thebwt
'''
import pyglet
from pyglet.graphics import vertex_list, Batch
from pyglet.gl import GL_QUADS, GL_LINE_LOOP, GL_LINES
from pyglet.text import Label

from wengine.widgets.buttons import ButtonList, Button, DirtyButtonList

class RootMenu(object):
	'''
	classdocs
	'''


	def __init__(self, label = "Wengine Base", buttons = (("PlaceHolder", None),)):
		'''
		Constructor
		'''
		#set up the label
		
		self.label = Label(label, color = (0,0,0,255), anchor_x = 'right', anchor_y = 'top', x =775, y = 575, font_size = 32)
		#set up the buttons
		self.buttons = ButtonList((25,300), buttons)
		
	def update(self):
		self.label.draw()
		self.buttons.draw()

class WindowMenu(object):
	'''
	class docs
	'''
	def __init__(self, title = None, rows = ((None,),), backgroundColor = (255,255,255), borderColor = (0,0,0), fontColor = (0,0,0,255), pos = (0,600), parent = None):
		self.title = title
		self.backgroundColor = backgroundColor
		self.borderColor = borderColor
		self.fontColor = fontColor
		self.x, self.y = pos
		self.width = 200
		self.height = 200
		self.closed = False
		self.parent = parent
		self.batch = Batch()
		self.separators = []
		self.fields = []
		
		y = self.y
		#set up Header
		if title:
			self.label = Label(self.title, color = (0,0,0,255), anchor_y = 'top', font_size = 16, batch = self.batch)
			self.label.y = pos[1] + self.label.content_height
			self.label.x = pos[0]
		#set up contents
		genButtons = []
		self.contents = []
		for row in rows:
			newRow = []
			x = self.x
			for item in row:
				if item:
					if item[0]=="label":
						newItem = MenuLabel(item[1][0], self.fontColor, batch = self.batch)
						newItem.x, newItem.y= x, y
						newRow.append(newItem)
						#print "label"
					elif item[0]=="entry":
						pass
					elif item[0]=="button":
						newItem = Button(item[1], batch = self.batch)
						newItem.x, newItem.y= x, y
						genButtons.append(newItem)
						newRow.append(newItem)
						#print "button"
					elif item[0]=="separator":
						pass
						newRow = False
						break
					elif item[0]=="field":
						newItem = FieldWidget(MenuLabel(item[1][0], self.fontColor, batch = self.batch))
						newItem.place(x, y)
						newRow.append(newItem)
						self.fields.append((item[1][0], newItem.getText))
						self.close=newItem.close(self.close)
						#genButtons.append(newItem)
					else:
						pass
				x = x + newItem.content_width
			if newRow: # not a separator
				y -= newRow[0].content_height
			else:
				self.separators.append(vertex_list(2,
					                          ('v2i', (x,y-2 ,x+200,y-2)),
					                          ('c3B', borderColor * 2)
					                          ))
				y-= 5
		#if genButtons: genButtons[0].func = self.close
		for button in genButtons:
			if button.func == "close-button": button.func = self.close
		self.buttons = DirtyButtonList(genButtons)
		#set up the background
		self.background = vertex_list(4,
			                     ('v2i', (self.x,self.y ,self.x + self.width,self.y ,self.x + self.width,self.y - self.height ,self.x,self.y - self.height)),
			                     ('c3B', backgroundColor * 4)
			                     )
		
		self.border = vertex_list(4,
			                 ('v2i', (self.x,self.y ,self.x + self.width,self.y ,self.x + self.width,self.y - self.height ,self.x,self.y - self.height)),
			                 ('c3B', borderColor * 4)
			                 )

	def close(self):
		self.parent.closeWindow(self)

	def draw(self):
		self.background.draw(GL_QUADS)
		self.border.draw(GL_LINE_LOOP)
		self.batch.draw()
		self.buttons.draw()
		for sep in self.separators:
			sep.draw(GL_LINES)

class MenuLabel(Label):
	'''
	class docs
	'''
	
	def __init__(self, text, inColor, inFont_size = 12, batch = None):
		super(MenuLabel, self).__init__(text, batch = batch, color = inColor, anchor_y = 'top')

class FieldWidget:
	'''class docs'''

	def __init__(self, label):
		self.label = label
		document = pyglet.text.document.UnformattedDocument()
		width = 200 - self.label.content_width
		self.field = pyglet.text.layout.IncrementalTextLayout(document, width, self.label.content_height, batch = self.label.batch)
		self.caret = pyglet.text.caret.Caret(self.field)
		self.content_width = self.label.content_width + self.field.width
		self.content_height = self.label.content_height
		self.caret.on_mouse_press = None
		for window in pyglet.app.windows:
			window.push_handlers(self.caret)

	def draw(self):
		self.label.draw()
		self.field.draw()
		self.caret.draw()

	def close(self, func):

		def pop_caret():
			for window in pyglet.app.windows:
				window.pop_handlers()

			func()

		return pop_caret

	def place(self, x, y):
		self.label.y = self.field.y = y
		self.label.anchor_y = self.field.anchor_y = "top"
		self.label.x = x
		self.field.x = x + self.label.content_width

	def getText(self):
		return self.field.document.text

class WindowForm(WindowMenu):

	def __init__(self, title = None, func = None,rows = ((None,),), backgroundColor = (255,255,255), borderColor = (0,0,0), fontColor = (0,0,0,255), pos = (0,600), parent = None):
		super(WindowForm, self).__init__(title, rows, backgroundColor, borderColor, fontColor, pos, parent)
		def form_function():
			mydict={}
			for kwarg in self.fields:
				mydict[kwarg[0]]=kwarg[1]()
			for window in pyglet.app.windows:
				window.pop_handlers()
			func(**mydict)
		for button in self.buttons.buttons:
			if button.func == "form-submit":
				button.func = form_function

def dud(self):
	pass
